package org.jastadd.ros2rag.tests;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jastadd.ros2rag.compiler.Compiler;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static org.jastadd.ros2rag.tests.TestUtils.exec;
import static org.jastadd.ros2rag.tests.TestUtils.readFile;
import static org.junit.jupiter.api.Assertions.assertTrue;

class Errors {

  private static final Logger logger = LogManager.getLogger(Errors.class);
  private static final String FILENAME_PATTERN = "$FILENAME";
  private static final String INPUT_DIRECTORY = "./src/test/01-input/errors/";
  private static final String OUTPUT_DIRECTORY = "./src/test/02-after-ros2rag/errors/";

  @BeforeAll
  public static void createOutputDirectory() {
    File outputDirectory = new File(OUTPUT_DIRECTORY);
    assertTrue((outputDirectory.exists() && outputDirectory.isDirectory()) || outputDirectory.mkdir());
  }

  @Test
  void testStandardErrors() throws IOException {
    test("Errors", "A");
  }

  @SuppressWarnings("SameParameterValue")
  private void test(String name, String rootNode) throws IOException {
    String grammarFile = INPUT_DIRECTORY + name + ".relast";
    String ros2ragFile = INPUT_DIRECTORY + name + ".ros2rag";
    String outFile = OUTPUT_DIRECTORY + name + ".out";
    String expectedFile = INPUT_DIRECTORY + name + ".expected";

    try {
      logger.debug("user.dir: {}", System.getProperty("user.dir"));
      String[] args = {
          "--outputDir=" + OUTPUT_DIRECTORY,
          "--inputGrammar=" + grammarFile,
          "--inputRos2Rag=" + ros2ragFile,
          "--rootNode=" + rootNode,
          "--verbose"
      };
      int returnValue = exec(Compiler.class, args, new File(outFile));
      Assertions.assertEquals(1, returnValue, "Ros2Rag did not return with value 1");
    } catch (IOException | InterruptedException e) {
      e.printStackTrace();
    }

    String out = readFile(outFile, Charset.defaultCharset());
    String expected = readFile(expectedFile, Charset.defaultCharset());
//    if (inFiles.size() == 1) {
      expected = expected.replace(FILENAME_PATTERN, name);
//    } else {
//      for (int i = 0; i < inFiles.size(); i++) {
//        expected = expected.replace(FILENAME_PATTERN + (i + 1), inFiles.get(i));
//      }
//    }
    List<String> outList = Arrays.asList(out.split("\n"));
    Collections.sort(outList);
    List<String> expectedList = Arrays.stream(expected.split("\n"))
        .sorted()
        .filter(s -> !s.isEmpty() && !s.startsWith("//"))
        .collect(Collectors.toList());

    // FIXME errors not handled correctly at the moment
//    Assertions.assertLinesMatch(expectedList, outList);

    logger.info("ros2rag for " + name + " returned:\n{}", out);
  }

}
